##############################################################################
##############################################################################
#                               Rizo.tcl
# In this file are implemented the procedures to actually do the downloading
# by executing 'curl'.
##############################################################################
# Copyright 1999 Andrs Garca Garca  -- ornalux@redestb.es
# Distributed under the terms of the GPL license
##############################################################################
##############################################################################
namespace eval Rizo {
variable errorMessage ""
variable meta
variable informe

##############################################################################
# RefererPage
#    It takes the path of the page from which the link was taken, as composes
#    its url. As filenames sometimes get changed, removing '~' for example,
#    the composed url may not be exactly the same, but if there is an
#    'file not found' error, it will be enough to clue the publisher, so
#    he can fix the error, that's the idea anyway.
#
# Parameter:
#    mother: full path to the referer page
#
# Returns
#    The url (or something like it)
##############################################################################
proc RefererPage {mother} {
    global direccion directorio_base

    if {![regexp {(?:.:)(.*)} $directorio_base nada base]} {
        set base $directorio_base
    }
    regexp -nocase {(.*?\..*?)(\.html)} $mother nada mother
    regexp "(?:$base)(.*)" $mother nada remotePath
    set url http://$direccion(www)$remotePath

    return $url
}

##############################################################################
# Comun
#    This procedure takes care of initializing the state variables an invoke
#    'curl' for all the connection types.
#
# Parmetros:
#    args: args that will be passed to curl
#    type: type of connection:
#           - cab: Headers
#           - dat: The link itself
##############################################################################
proc Comun {args type} {
    variable error 0
    variable informe
    variable rizo
    variable meta
    variable pipe
    global opciones errorCode

    set informe(largo)          0
    set informe(detener)        0
    set informe(largo)          0
    set informe(velocidad)      0
    set informe(parar)          0
    set informe(vanBytes)       0
    set curlError               0
    set errorCode               ""

    if {$opciones(proxy)==1} {
        set curlCmd [concat curl -x $opciones(dirProxy) $args]
    } else {
        set curlCmd [concat curl $args]
    }
#puts $curlCmd
    eval {set pipe [open "| $curlCmd |& cat" r]}

    fileevent $pipe readable [list [namespace code Lector] $pipe $type]
    fconfigure $pipe -blocking 0

    return
}

###############################################################################
# Cabeza
#    Asks the server for the Headers of the link
#
# Parameters:
#    link: url to download
#    mother: referer page of the link
###############################################################################
proc Cabeza {link mother} {
    variable rizo
    variable meta
    variable informe
# It seems some servers return the body on a HEAD request
    variable workaround

    set meta(contenido) ""
    set meta(relocate)  ""
    set workaround 0

    if [info exists meta(totalBytes)] {
        unset meta(totalBytes)
    }

    if {$mother!="-"} {
        set refererUrl [RefererPage $mother]
        set args [list -e $refererUrl -I $link]
    } else {
        set args [list -I $link]
    }
    Comun $args cab

    return
}

###############################################################################
# Datos
#   Downloads a given url
#
# Parameters:
#   file:  file in which the url will be saved
#   linke: url to download
#   mother: referer url of the link
###############################################################################
proc Datos {file link mother} {
    variable rizo
    variable informe
    variable meta
    global opciones

    set informe(porcentaje) 0
    if {![info exists meta(totalBytes)]} {
        set meta(totalBytes) 0
    }

    if {$mother!="-"} {
        set refererUrl [RefererPage $mother]
        set args [list -e $refererUrl -o $file $link]
    } else {
        set args [list -o $file $link]
    }

    Comun $args dat

    return
}

###############################################################################
# Reanudar:
#    Resumes, server allowing, a download
#
# Parameters:
#    file: full path of the file where the url will be downloaded
#    link: url to download
#    mother: referer page of the url
###############################################################################
proc Reanudar {file link mother} {
    variable rizo
    variable informe
    variable meta

    set informe(porcentaje) 0
    if {![info exists meta(totalBytes)]} {
        set meta(totalBytes) 0
    }

    if {$mother!="-"} {
        set refererUrl [RefererPage $mother]
        set args [list -e $refererUrl -c -o $file $link]
    } else {
        set args [list -c -o $file $link]
    }
    Comun $args dat

    return
}

###############################################################################
# Lector
#   This procedure controls the downloading, it is invoked anytime there is
#   something to proccess
#
# Parameters:
#   pipe: pipe used to invoke 'curl'
#   tipo: type of request (HEAD, GET, ...) or stopNow, to stop
###############################################################################
proc Lector {pipe tipo} {
    variable meta
    variable informe
    variable curlError
    variable workaround
    global errorCode getleftState

    if {($tipo=="stopNow")||($getleftState(downloading)==0)} {
        catch {close $pipe}
        return
    }
    if {[catch "eof $pipe" endOfFile]} {
        tk_messageBox -title "Error" -messageBox "Could you please e-mail me\n \
                saying you saw this?" -type ok
        return
    }
    if {($endOfFile) || ($informe(parar)==1) || ($informe(detener)==1)} {
        if {($endOfFile)} {
            set informe(fin) 1
        }
        catch {close $pipe}
        set curlError [lindex $errorCode 2]
#        if {$curlError!=""} {
#            puts "Cdigo de error: $curlError - $errorCode"
#        }
        return
    }
    if {[gets $pipe line]>=0} {
        if {[string first curl $line]!=-1} {
            set Ventana::Rizo::errorMessage $line
            return
        }
        if [string match $line ""] return
        if {$tipo=="cab"} {
            if {([regexp {^<} $line])||($workaround==1)} {
                set workaround 1
                return
            }
#puts $line
            if [regexp -nocase {^(?:HTTP/)([0-9].[0-9])(?:\ )([0-9]*)(?:\ )(.*)} \
                    $line nada meta(versionServer) meta(codigo) meta(error)] {
                if {$meta(codigo)>=400} {
                    set Ventana::Rizo::errorMessage $meta(error)
                    catch {error "Server Error" SERVER \
                            "Server \"$meta(error)\" $meta(codigo)"}
                    return
                }
            }
            regexp -nocase {^(?:Location: )(.*)}      $line nada meta(relocate)
            regexp -nocase {^(?:Date: )(.*)}          $line nada meta(fecha)
            regexp -nocase {^(?:Content-Type: )(.*)}  $line nada meta(contenido)
            regexp -nocase {^(?:Last-Modified: )(.*)} $line nada meta(cambio)
            regexp -nocase {^(?:Content-Length: )(.*)} \
                    $line nada meta(totalBytes)
        } else {
            set informe(velocidad) 0
            set informe(vanBytes)  0

            if {[string first % $line]!=-1} {
                set informe(largo) 1
                return
            }
            if {$informe(largo)==1} {
                set informe(porcentaje) [lindex $line 0]
                set informe(velocidad)  [lindex $line 6]
            } else {
                regexp {(^.*?)(?: )(?:.*\()(.*)( )} $line \
                        nada informe(vanBytes) informe(velocidad)
                if [catch {expr $informe(vanBytes)*100.0/$meta(totalBytes)} \
                        informe(porcentaje)] {
                    set errorCode ""
                    set informe(porcentaje) 0
                } else {
                    set informe(porcentaje) [expr round($informe(porcentaje))]
                }
                if {$informe(vanBytes)<512} {
                    set informe(vanBytes) \
                        "[format "%.0f" $informe(vanBytes)] bytes"
                } elseif {$informe(vanBytes)<1048576} {
                    set informe(vanBytes) \
                        "[format "%.2f" [expr $informe(vanBytes)/1024.0]] k"
                } else {
                    set informe(vanBytes) \
                        "[format "%.2f" [expr $informe(vanBytes)/1048576.0]] M"
                }
            }
            if {![regexp {/} $informe(velocidad)]} {
                if {($informe(velocidad)>512)} {
                    set informe(velocidad) \
                        "[format "%.2f" [expr $informe(velocidad)/1024.0]] k/s"
                } else {
                    set informe(velocidad) \
                        "[format "%.2f" $informe(velocidad)] bytes/s"
                }
            }
            if {$informe(largo)==1} {
                set informe(queda) "[lindex $line 4] ( $informe(velocidad) )"
            } else {
                set informe(queda) "$informe(vanBytes) ( $informe(velocidad) )"
            }
        }
    }
    return
}

}
